@@ -1,7 +1,7 @@
 /* pop.c: client routines for talking to a POP3-protocol post-office server
 
-Copyright (C) 1991, 1993, 1996-1997, 1999, 2001-2012
-  Free Software Foundation, Inc.
+Copyright (C) 1991, 1993, 1996-1997, 1999, 2001-2018 Free Software
+Foundation, Inc.
 
 Author: Jonathan Kamens <jik@security.ov.com>
 
@@ -9,8 +9,8 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+the Free Software Foundation, either version 3 of the License, or (at
+your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,21 +18,27 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
+along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#else
-#define MAIL_USE_POP
-#endif
 
 #ifdef MAIL_USE_POP
 
 #include <sys/types.h>
 #ifdef WINDOWSNT
 #include "ntlib.h"
-#include <winsock.h>
+#undef _WIN32_WINNT
+#define _WIN32_WINNT 0x0501	/* for getaddrinfo stuff */
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#undef getaddrinfo
+#define getaddrinfo  sys_getaddrinfo
+#undef freeaddrinfo
+#define freeaddrinfo sys_freeaddrinfo
+int sys_getaddrinfo (const char * node, const char * service,
+		     const struct addrinfo * hints, struct addrinfo ** res);
+void sys_freeaddrinfo (struct addrinfo * ai);
 #undef SOCKET_ERROR
 #define RECV(s,buf,len,flags) recv (s,buf,len,flags)
 #define SEND(s,buf,len,flags) send (s,buf,len,flags)
@@ -46,10 +52,6 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 #endif
 #include <pop.h>
 
-#ifdef sun
-#include <malloc.h>
-#endif /* sun */
-
 #ifdef HESIOD
 #include <hesiod.h>
 /*
@@ -61,10 +63,12 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 extern struct servent *hes_getservbyname (/* char *, char * */);
 #endif
 
+#include <alloca.h>
 #include <pwd.h>
 #include <netdb.h>
 #include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -88,6 +92,7 @@ extern struct servent *hes_getservbyname (/* char *, char * */);
 # endif
 #endif /* KERBEROS */
 
+#include <c-ctype.h>
 #include <min-max.h>
 
 #ifdef KERBEROS
@@ -100,12 +105,6 @@ extern char *krb_realmofhost (/* char * */);
 #endif /* ! KERBEROS5 */
 #endif /* KERBEROS */
 
-#ifndef WINDOWSNT
-#if !defined (HAVE_H_ERRNO) || !defined (HAVE_CONFIG_H)
-extern int h_errno;
-#endif
-#endif
-
 static int socket_connection (char *, int);
 static int pop_getline (popserver, char **);
 static int sendline (popserver, const char *);
@@ -128,7 +127,7 @@ static char *find_crlf (char *, int);
 #endif
 
 char pop_error[ERROR_MAX];
-int pop_debug = 0;
+bool pop_debug = false;
 
 /*
  * Function: pop_open (char *host, char *username, char *password,
@@ -273,8 +272,8 @@ pop_open (char *host, char *username, char *password, int flags)
   server->data = 0;
   server->buffer_index = 0;
   server->buffer_size = GETLINE_MIN;
-  server->in_multi = 0;
-  server->trash_started = 0;
+  server->in_multi = false;
+  server->trash_started = false;
 
   if (getok (server))
     return (0);
@@ -344,10 +343,7 @@ pop_stat (popserver server, int *count, int *size)
   if (strncmp (fromserver, "+OK ", 4))
     {
       if (0 == strncmp (fromserver, "-ERR", 4))
-	{
-	  strncpy (pop_error, fromserver, ERROR_MAX);
-	  pop_error[ERROR_MAX-1] = '\0';
-	}
+	snprintf (pop_error, ERROR_MAX, "%s", fromserver);
       else
 	{
 	  strcpy (pop_error,
@@ -448,10 +444,7 @@ pop_list (popserver server, int message, int **IDs, int **sizes)
       if (strncmp (fromserver, "+OK ", 4))
 	{
 	  if (! strncmp (fromserver, "-ERR", 4))
-	    {
-	      strncpy (pop_error, fromserver, ERROR_MAX);
-	      pop_error[ERROR_MAX-1] = '\0';
-	    }
+	    snprintf (pop_error, ERROR_MAX, "%s", fromserver);
 	  else
 	    {
 	      strcpy (pop_error,
@@ -690,14 +683,13 @@ pop_multi_first (popserver server, const char *command, char **response)
 
   if (0 == strncmp (*response, "-ERR", 4))
     {
-      strncpy (pop_error, *response, ERROR_MAX);
-      pop_error[ERROR_MAX-1] = '\0';
+      snprintf (pop_error, ERROR_MAX, "%s", *response);
       return (-1);
     }
   else if (0 == strncmp (*response, "+OK", 3))
     {
       for (*response += 3; **response == ' '; (*response)++) /* empty */;
-      server->in_multi = 1;
+      server->in_multi = true;
       return (0);
     }
   else
@@ -729,7 +721,8 @@ pop_multi_next (popserver server, char **line)
       return (-1);
     }
 
-  if ((ret = pop_getline (server, &fromserver)) < 0)
+  ret = pop_getline (server, &fromserver);
+  if (ret < 0)
     {
       return (-1);
     }
@@ -739,7 +732,7 @@ pop_multi_next (popserver server, char **line)
       if (! fromserver[1])
 	{
 	  *line = 0;
-	  server->in_multi = 0;
+	  server->in_multi = false;
 	  return (0);
 	}
       else
@@ -864,8 +857,7 @@ pop_last (popserver server)
 
   if (! strncmp (fromserver, "-ERR", 4))
     {
-      strncpy (pop_error, fromserver, ERROR_MAX);
-      pop_error[ERROR_MAX-1] = '\0';
+      snprintf (pop_error, ERROR_MAX, "%s", fromserver);
       return (-1);
     }
   else if (strncmp (fromserver, "+OK ", 4))
@@ -978,13 +970,9 @@ static int have_winsock = 0;
 static int
 socket_connection (char *host, int flags)
 {
-#ifdef HAVE_GETADDRINFO
   struct addrinfo *res, *it;
   struct addrinfo hints;
   int ret;
-#else /* !HAVE_GETADDRINFO */
-  struct hostent *hostent;
-#endif
   struct servent *servent;
   struct sockaddr_in addr;
   char found_port = 0;
@@ -1065,14 +1053,12 @@ socket_connection (char *host, int flags)
   sock = socket (PF_INET, SOCK_STREAM, 0);
   if (sock < 0)
     {
-      strcpy (pop_error, POP_SOCKET_ERROR);
-      strncat (pop_error, strerror (errno),
-	       ERROR_MAX - sizeof (POP_SOCKET_ERROR));
+      snprintf (pop_error, ERROR_MAX, "%s%s",
+		POP_SOCKET_ERROR, strerror (errno));
       return (-1);
 
     }
 
-#ifdef HAVE_GETADDRINFO
   memset (&hints, 0, sizeof (hints));
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_flags = AI_CANONNAME;
@@ -1088,64 +1074,28 @@ socket_connection (char *host, int flags)
 	}
     } while (ret != 0);
 
-  if (ret == 0)
+  for (it = res; it; it = it->ai_next)
+    if (it->ai_addrlen == sizeof addr)
+      {
+	struct sockaddr_in *in_a = (struct sockaddr_in *) it->ai_addr;
+	addr.sin_addr = in_a->sin_addr;
+	if (! connect (sock, (struct sockaddr *) &addr, sizeof addr))
+	  break;
+      }
+  connect_ok = it != NULL;
+  if (connect_ok)
     {
-      it = res;
-      while (it)
-        {
-          if (it->ai_addrlen == sizeof (addr))
-            {
-              struct sockaddr_in *in_a = (struct sockaddr_in *) it->ai_addr;
-              memcpy (&addr.sin_addr, &in_a->sin_addr, sizeof (addr.sin_addr));
-              if (! connect (sock, (struct sockaddr *) &addr, sizeof (addr)))
-                break;
-            }
-          it = it->ai_next;
-        }
-      connect_ok = it != NULL;
-      if (connect_ok)
-        {
-          realhost = alloca (strlen (it->ai_canonname) + 1);
-          strcpy (realhost, it->ai_canonname);
-        }
-      freeaddrinfo (res);
+      realhost = alloca (strlen (it->ai_canonname) + 1);
+      strcpy (realhost, it->ai_canonname);
     }
-#else /* !HAVE_GETADDRINFO */
-  do
-    {
-      hostent = gethostbyname (host);
-      try_count++;
-      if ((! hostent) && ((h_errno != TRY_AGAIN) || (try_count == 5)))
-	{
-	  strcpy (pop_error, "Could not determine POP server's address");
-	  return (-1);
-	}
-    } while (! hostent);
-
-  while (*hostent->h_addr_list)
-    {
-      memcpy (&addr.sin_addr, *hostent->h_addr_list, hostent->h_length);
-      if (! connect (sock, (struct sockaddr *) &addr, sizeof (addr)))
-	break;
-      hostent->h_addr_list++;
-    }
-  connect_ok = *hostent->h_addr_list != NULL;
-  if (! connect_ok)
-    {
-      realhost = alloca (strlen (hostent->h_name) + 1);
-      strcpy (realhost, hostent->h_name);
-    }
-
-#endif /* !HAVE_GETADDRINFO */
+  freeaddrinfo (res);
 
 #define CONNECT_ERROR "Could not connect to POP server: "
 
   if (! connect_ok)
     {
       CLOSESOCKET (sock);
-      strcpy (pop_error, CONNECT_ERROR);
-      strncat (pop_error, strerror (errno),
-	       ERROR_MAX - sizeof (CONNECT_ERROR));
+      snprintf (pop_error, ERROR_MAX, "%s%s", CONNECT_ERROR, strerror (errno));
       return (-1);
 
     }
@@ -1156,43 +1106,43 @@ socket_connection (char *host, int flags)
   if (! (flags & POP_NO_KERBEROS))
     {
 #ifdef KERBEROS5
-      if ((rem = krb5_init_context (&kcontext)))
+      rem = krb5_init_context (&kcontext);
+      if (rem)
 	{
 	krb5error:
 	  if (auth_context)
 	    krb5_auth_con_free (kcontext, auth_context);
 	  if (kcontext)
 	    krb5_free_context (kcontext);
-	  strcpy (pop_error, KRB_ERROR);
-	  strncat (pop_error, error_message (rem),
-		   ERROR_MAX - sizeof (KRB_ERROR));
+	  snprintf (pop_error, ERROR_MAX, "%s%s",
+		    KRB_ERROR, error_message (rem));
 	  CLOSESOCKET (sock);
 	  return (-1);
 	}
 
-      if ((rem = krb5_auth_con_init (kcontext, &auth_context)))
+      rem = krb5_auth_con_init (kcontext, &auth_context);
+      if (rem)
 	goto krb5error;
 
-      if (rem = krb5_cc_default (kcontext, &ccdef))
+      rem = krb5_cc_default (kcontext, &ccdef);
+      if (rem)
 	goto krb5error;
 
-      if (rem = krb5_cc_get_principal (kcontext, ccdef, &client))
+      rem = krb5_cc_get_principal (kcontext, ccdef, &client);
+      if (rem)
 	goto krb5error;
 
       for (cp = realhost; *cp; cp++)
-	{
-	  if (isupper (*cp))
-	    {
-	      *cp = tolower (*cp);
-	    }
-	}
+	*cp = c_tolower (*cp);
 
-      if (rem = krb5_sname_to_principal (kcontext, realhost,
-					 POP_SERVICE, FALSE, &server))
+      rem = krb5_sname_to_principal (kcontext, realhost,
+				     POP_SERVICE, FALSE, &server);
+      if (rem)
 	goto krb5error;
 
       rem = krb5_sendauth (kcontext, &auth_context,
-			   (krb5_pointer) &sock, "KPOPV1.0", client, server,
+			   (krb5_pointer) &sock, (char *) "KPOPV1.0",
+			   client, server,
 			  AP_OPTS_MUTUAL_REQUIRED,
 			  0,	/* no checksum */
 			  0,	/* no creds, use ccache instead */
@@ -1203,30 +1153,19 @@ socket_connection (char *host, int flags)
       krb5_free_principal (kcontext, server);
       if (rem)
 	{
-	  strcpy (pop_error, KRB_ERROR);
-	  strncat (pop_error, error_message (rem),
-		   ERROR_MAX - sizeof (KRB_ERROR));
+	  int pop_error_len = snprintf (pop_error, ERROR_MAX, "%s%s",
+					KRB_ERROR, error_message (rem));
 #if defined HAVE_KRB5_ERROR_TEXT
 	  if (err_ret && err_ret->text.length)
 	    {
-	      strncat (pop_error, " [server says '",
-		       ERROR_MAX - strlen (pop_error) - 1);
-	      strncat (pop_error, err_ret->text.data,
-		       min (ERROR_MAX - strlen (pop_error) - 1,
-			    err_ret->text.length));
-	      strncat (pop_error, "']",
-		       ERROR_MAX - strlen (pop_error) - 1);
+	      int errlen = err_ret->text.length;
+	      snprintf (pop_error + pop_error_len, ERROR_MAX - pop_error_len,
+			" [server says '%.*s']", errlen, err_ret->text.data);
 	    }
 #elif defined HAVE_KRB5_ERROR_E_TEXT
-	  if (err_ret && err_ret->e_text && strlen (*err_ret->e_text))
-	    {
-	      strncat (pop_error, " [server says '",
-		       ERROR_MAX - strlen (pop_error) - 1);
-	      strncat (pop_error, *err_ret->e_text,
-		       ERROR_MAX - strlen (pop_error) - 1);
-	      strncat (pop_error, "']",
-		       ERROR_MAX - strlen (pop_error) - 1);
-	    }
+	  if (err_ret && err_ret->e_text && **err_ret->e_text)
+	    snprintf (pop_error + pop_error_len, ERROR_MAX - pop_error_len,
+		      " [server says '%s']", *err_ret->e_text);
 #endif
 	  if (err_ret)
 	    krb5_free_error (kcontext, err_ret);
@@ -1247,9 +1186,7 @@ socket_connection (char *host, int flags)
       free ((char *) ticket);
       if (rem != KSUCCESS)
 	{
-	  strcpy (pop_error, KRB_ERROR);
-	  strncat (pop_error, krb_err_txt[rem],
-		   ERROR_MAX - sizeof (KRB_ERROR));
+	  snprintf (pop_error, ERROR_MAX, "%s%s", KRB_ERROR, krb_err_txt[rem]);
 	  CLOSESOCKET (sock);
 	  return (-1);
 	}
@@ -1354,9 +1291,8 @@ pop_getline (popserver server, char **line)
 		  server->buffer_size - server->data - 1, 0);
       if (ret < 0)
 	{
-	  strcpy (pop_error, GETLINE_ERROR);
-	  strncat (pop_error, strerror (errno),
-		   ERROR_MAX - sizeof (GETLINE_ERROR));
+	  snprintf (pop_error, ERROR_MAX, "%s%s",
+		    GETLINE_ERROR, strerror (errno));
 	  pop_trash (server);
 	  return (-1);
 	}
@@ -1433,16 +1369,13 @@ sendline (popserver server, const char *line)
      over a few dozen messages, and is a big chunk of the time we
      spend fetching mail from a server close by.  */
   buf = alloca (strlen (line) + 3);
-  strcpy (buf, line);
-  strcat (buf, "\r\n");
+  strcpy (stpcpy (buf, line), "\r\n");
   ret = fullwrite (server->file, buf, strlen (buf));
 
   if (ret < 0)
     {
       pop_trash (server);
-      strcpy (pop_error, SENDLINE_ERROR);
-      strncat (pop_error, strerror (errno),
-	       ERROR_MAX - sizeof (SENDLINE_ERROR));
+      snprintf (pop_error, ERROR_MAX, "%s%s", SENDLINE_ERROR, strerror (errno));
       return (ret);
     }
 
@@ -1504,8 +1437,7 @@ getok (popserver server)
     return (0);
   else if (! strncmp (fromline, "-ERR", 4))
     {
-      strncpy (pop_error, fromline, ERROR_MAX);
-      pop_error[ERROR_MAX-1] = '\0';
+      snprintf (pop_error, ERROR_MAX, "%s", fromline);
       return (-1);
     }
   else
@@ -1585,7 +1517,7 @@ pop_trash (popserver server)
       /* avoid recursion; sendline can call pop_trash */
       if (server->trash_started)
 	return;
-      server->trash_started = 1;
+      server->trash_started = true;
 
       sendline (server, "RSET");
       sendline (server, "QUIT");
@@ -1625,4 +1557,143 @@ find_crlf (char *in_string, int len)
   return (0);
 }
 
+#ifdef WINDOWSNT
+/* The following 2 functions are only available since XP, so we load
+   them dynamically and provide fallbacks.  */
+
+int (WINAPI *pfn_getaddrinfo) (const char *, const char *,
+			       const struct addrinfo *, struct addrinfo **);
+void (WINAPI *pfn_freeaddrinfo) (struct addrinfo *);
+
+static int
+load_ws2 (void)
+{
+  static int ws2_loaded = 0;
+
+  if (!ws2_loaded)
+    {
+      HANDLE ws2_lib = LoadLibrary ("Ws2_32.dll");
+
+      if (ws2_lib != NULL)
+	{
+	  ws2_loaded = 1;
+	  pfn_getaddrinfo = (void *) GetProcAddress (ws2_lib, "getaddrinfo");
+	  pfn_freeaddrinfo = (void *) GetProcAddress (ws2_lib, "freeaddrinfo");
+	  /* Paranoia: these two functions should go together, so if
+	     one is absent, we cannot use the other.  */
+	  if (pfn_getaddrinfo == NULL)
+	    pfn_freeaddrinfo = NULL;
+	  else if (pfn_freeaddrinfo == NULL)
+	    pfn_getaddrinfo = NULL;
+	}
+    }
+  if (!ws2_loaded)
+    {
+      errno = ENETDOWN;
+      return -1;
+    }
+  return 0;
+}
+
+
+int
+sys_getaddrinfo (const char *node, const char *service,
+		 const struct addrinfo *hints, struct addrinfo **res)
+{
+  int rc;
+
+  if (load_ws2 () != 0)
+    {
+      errno = ENETDOWN;
+      return WSANO_RECOVERY;
+    }
+
+  if (pfn_getaddrinfo)
+    rc = pfn_getaddrinfo (node, service, hints, res);
+  else
+    {
+      int port = 0;
+      struct hostent *host_info;
+      struct gai_storage {
+	struct addrinfo addrinfo;
+	struct sockaddr_in sockaddr_in;
+      } *gai_storage;
+
+      /* We don't support any flags besides AI_CANONNAME.  */
+      if (hints && (hints->ai_flags & ~(AI_CANONNAME)) != 0)
+	return WSAEINVAL;
+      /* NODE cannot be NULL, since pop.c has fallbacks for that.  */
+      if (!node)
+	return WSAHOST_NOT_FOUND;
+
+      if (service)
+	{
+	  const char *protocol =
+	    (hints && hints->ai_socktype == SOCK_DGRAM) ? "udp" : "tcp";
+	  struct servent *srv = getservbyname (service, protocol);
+
+	  if (srv)
+	    port = srv->s_port;
+	  else
+	    return WSAHOST_NOT_FOUND;
+	}
+
+      gai_storage = calloc (1, sizeof *gai_storage);
+      gai_storage->sockaddr_in.sin_port = port;
+      host_info = gethostbyname (node);
+      if (host_info)
+	{
+	  memcpy (&gai_storage->sockaddr_in.sin_addr,
+		  host_info->h_addr, host_info->h_length);
+	  gai_storage->sockaddr_in.sin_family = host_info->h_addrtype;
+	}
+      else
+	{
+	  free (gai_storage);
+	  return WSAHOST_NOT_FOUND;
+	}
+
+      gai_storage->addrinfo.ai_addr =
+	(struct sockaddr *)&gai_storage->sockaddr_in;
+      gai_storage->addrinfo.ai_addrlen = sizeof (gai_storage->sockaddr_in);
+      if (hints && (hints->ai_flags & AI_CANONNAME) != 0)
+	{
+	  gai_storage->addrinfo.ai_canonname = strdup (host_info->h_name);
+	  if (!gai_storage->addrinfo.ai_canonname)
+	    {
+	      free (gai_storage);
+	      return WSA_NOT_ENOUGH_MEMORY;
+	    }
+	}
+      gai_storage->addrinfo.ai_protocol = (hints) ? hints->ai_protocol : 0;
+      gai_storage->addrinfo.ai_socktype = (hints) ? hints->ai_socktype : 0;
+      gai_storage->addrinfo.ai_family = gai_storage->sockaddr_in.sin_family;
+      gai_storage->addrinfo.ai_next = NULL;
+
+      *res = &gai_storage->addrinfo;
+      rc = 0;
+    }
+
+  return rc;
+}
+
+void
+sys_freeaddrinfo (struct addrinfo *ai)
+{
+  if (load_ws2 () != 0)
+    {
+      errno = ENETDOWN;
+      return;
+    }
+
+  if (pfn_freeaddrinfo)
+    pfn_freeaddrinfo (ai);
+  else
+    {
+      if (ai->ai_canonname)
+	free (ai->ai_canonname);
+      free (ai);
+    }
+}
+#endif	/* WINDOWSNT */
 #endif /* MAIL_USE_POP */
