@@ -3,7 +3,7 @@
 ;;; Code:
 
 
-;;;### (autoloads nil "cl-extra" "cl-extra.el" "2c37fbacf9ba6947d5d6eb2d36ba5638")
+;;;### (autoloads nil "cl-extra" "cl-extra.el" "e04137603e1d8932609149e027be03e6")
 ;;; Generated autoloads from cl-extra.el
 
 (autoload 'cl-coerce "cl-extra" "\
@@ -23,7 +23,7 @@ strings case-insensitively.
 (autoload 'cl--mapcar-many "cl-extra" "\
 
 
-\(fn CL-FUNC CL-SEQS)" nil nil)
+\(fn CL-FUNC CL-SEQS &optional ACC)" nil nil)
 
 (autoload 'cl-map "cl-extra" "\
 Map a FUNCTION across one or more SEQUENCEs, returning a sequence.
@@ -153,6 +153,17 @@ Return 1 if X is positive, -1 if negative, 0 if zero.
 
 \(fn X)" nil nil)
 
+(autoload 'cl-parse-integer "cl-extra" "\
+Parse integer from the substring of STRING from START to END.
+STRING may be surrounded by whitespace chars (chars with syntax ` ').
+Other non-digit chars are considered junk.
+RADIX is an integer between 2 and 36, the default is 10.  Signal
+an error if the substring between START and END cannot be parsed
+as an integer unless JUNK-ALLOWED is non-nil.
+
+\(fn STRING &key START END RADIX JUNK-ALLOWED)" nil nil)
+ (autoload 'cl-random-state-p "cl-extra")
+
 (autoload 'cl-random "cl-extra" "\
 Return a random nonnegative number less than LIM, an integer or float.
 Optional second arg STATE is a random-state object.
@@ -165,11 +176,6 @@ If STATE is t, return a new state object seeded from the time of day.
 
 \(fn &optional STATE)" nil nil)
 
-(autoload 'cl-random-state-p "cl-extra" "\
-Return t if OBJECT is a random-state object.
-
-\(fn OBJECT)" nil nil)
-
 (autoload 'cl-float-limits "cl-extra" "\
 Initialize the Common Lisp floating-point parameters.
 This sets the values of: `cl-most-positive-float', `cl-most-negative-float',
@@ -183,6 +189,8 @@ This sets the values of: `cl-most-positive-float', `cl-most-negative-float',
 Return the subsequence of SEQ from START to END.
 If END is omitted, it defaults to the length of the sequence.
 If START or END is negative, it counts from the end.
+Signal an error if START or END are outside of the sequence (i.e
+too large if positive or too small if negative).
 
 \(fn SEQ START &optional END)" nil nil)
 
@@ -216,7 +224,7 @@ Return the value of SYMBOL's PROPNAME property, or DEFAULT if none.
 
 \(fn SYMBOL PROPNAME &optional DEFAULT)" nil nil)
 
-(eval-and-compile (put 'cl-get 'compiler-macro #'cl--compiler-macro-get))
+(eval-and-compile (function-put 'cl-get 'compiler-macro #'cl--compiler-macro-get))
 
 (autoload 'cl-getf "cl-extra" "\
 Search PROPLIST for property PROPNAME; return its value or DEFAULT.
@@ -239,6 +247,11 @@ Remove from SYMBOL's plist the property PROPNAME and its value.
 
 \(fn SYMBOL PROPNAME)" nil nil)
 
+(autoload 'cl-fresh-line "cl-extra" "\
+Output a newline unless already at the beginning of a line.
+
+\(fn &optional STREAM)" nil nil)
+
 (autoload 'cl-prettyexpand "cl-extra" "\
 Expand macros in FORM and insert the pretty-printed result.
 Optional argument FULL non-nil means to expand all macros,
@@ -246,9 +259,19 @@ including `cl-block' and `cl-eval-when'.
 
 \(fn FORM &optional FULL)" nil nil)
 
+(autoload 'cl-find-class "cl-extra" "\
+
+
+\(fn TYPE)" nil nil)
+
+(autoload 'cl-describe-type "cl-extra" "\
+Display the documentation for type TYPE (a symbol).
+
+\(fn TYPE)" t nil)
+
 ;;;***
 
-;;;### (autoloads nil "cl-macs" "cl-macs.el" "fe8a91b987c0ab7cad9fb8c12efd34a8")
+;;;### (autoloads nil "cl-macs" "cl-macs.el" "511d3aa68850bc44ad757bf25e6db152")
 ;;; Generated autoloads from cl-macs.el
 
 (autoload 'cl--compiler-macro-list* "cl-macs" "\
@@ -256,10 +279,7 @@ including `cl-block' and `cl-eval-when'.
 
 \(fn FORM ARG &rest OTHERS)" nil nil)
 
-(autoload 'cl--compiler-macro-cXXr "cl-macs" "\
-
-
-\(fn FORM X)" nil nil)
+(define-obsolete-function-alias 'cl--compiler-macro-cXXr 'internal--compiler-macro-cXXr "25.1")
 
 (autoload 'cl-gensym "cl-macs" "\
 Generate a new uninterned symbol.
@@ -269,7 +289,7 @@ The name is made by appending a number to PREFIX, default \"G\".
 
 (autoload 'cl-gentemp "cl-macs" "\
 Generate a new interned symbol with a unique name.
-The name is made by appending a number to PREFIX, default \"G\".
+The name is made by appending a number to PREFIX, default \"T\".
 
 \(fn &optional PREFIX)" nil nil)
 
@@ -278,22 +298,62 @@ Define NAME as a function.
 Like normal `defun', except ARGLIST allows full Common Lisp conventions,
 and BODY is implicitly surrounded by (cl-block NAME ...).
 
+The full form of a Common Lisp function argument list is
+
+   (VAR...
+    [&optional (VAR [INITFORM [SVAR]])...]
+    [&rest|&body VAR]
+    [&key (([KEYWORD] VAR) [INITFORM [SVAR]])... [&allow-other-keys]]
+    [&aux (VAR [INITFORM])...])
+
+VAR may be replaced recursively with an argument list for
+destructuring, `&whole' is supported within these sublists.  If
+SVAR, INITFORM, and KEYWORD are all omitted, then `(VAR)' may be
+written simply `VAR'.  See the Info node `(cl)Argument Lists' for
+more details.
+
 \(fn NAME ARGLIST [DOCSTRING] BODY...)" nil t)
 
-(put 'cl-defun 'doc-string-elt '3)
+(function-put 'cl-defun 'doc-string-elt '3)
 
-(put 'cl-defun 'lisp-indent-function '2)
+(function-put 'cl-defun 'lisp-indent-function '2)
+
+(autoload 'cl-iter-defun "cl-macs" "\
+Define NAME as a generator function.
+Like normal `iter-defun', except ARGLIST allows full Common Lisp conventions,
+and BODY is implicitly surrounded by (cl-block NAME ...).
+
+\(fn NAME ARGLIST [DOCSTRING] BODY...)" nil t)
+
+(function-put 'cl-iter-defun 'doc-string-elt '3)
+
+(function-put 'cl-iter-defun 'lisp-indent-function '2)
 
 (autoload 'cl-defmacro "cl-macs" "\
 Define NAME as a macro.
 Like normal `defmacro', except ARGLIST allows full Common Lisp conventions,
 and BODY is implicitly surrounded by (cl-block NAME ...).
 
+The full form of a Common Lisp macro argument list is
+
+   (VAR...
+    [&optional (VAR [INITFORM [SVAR]])...]
+    [&rest|&body VAR]
+    [&key (([KEYWORD] VAR) [INITFORM [SVAR]])... [&allow-other-keys]]
+    [&aux (VAR [INITFORM])...]
+    [&environment VAR])
+
+VAR may be replaced recursively with an argument list for
+destructuring, `&whole' is supported within these sublists.  If
+SVAR, INITFORM, and KEYWORD are all omitted, then `(VAR)' may be
+written simply `VAR'.  See the Info node `(cl)Argument Lists' for
+more details.
+
 \(fn NAME ARGLIST [DOCSTRING] BODY...)" nil t)
 
-(put 'cl-defmacro 'doc-string-elt '3)
+(function-put 'cl-defmacro 'doc-string-elt '3)
 
-(put 'cl-defmacro 'lisp-indent-function '2)
+(function-put 'cl-defmacro 'lisp-indent-function '2)
 
 (autoload 'cl-function "cl-macs" "\
 Introduce a function.
@@ -307,7 +367,7 @@ Bind the variables in ARGS to the result of EXPR and execute BODY.
 
 \(fn ARGS EXPR &rest BODY)" nil t)
 
-(put 'cl-destructuring-bind 'lisp-indent-function '2)
+(function-put 'cl-destructuring-bind 'lisp-indent-function '2)
 
 (autoload 'cl-eval-when "cl-macs" "\
 Control when BODY is evaluated.
@@ -317,7 +377,7 @@ If `eval' is in WHEN, BODY is evaluated when interpreted or at non-top-level.
 
 \(fn (WHEN...) BODY...)" nil t)
 
-(put 'cl-eval-when 'lisp-indent-function '1)
+(function-put 'cl-eval-when 'lisp-indent-function '1)
 
 (autoload 'cl-load-time-value "cl-macs" "\
 Like `progn', but evaluates the body at load time.
@@ -327,16 +387,17 @@ The result of the body appears to the compiler as a quoted constant.
 
 (autoload 'cl-case "cl-macs" "\
 Eval EXPR and choose among clauses on that value.
-Each clause looks like (KEYLIST BODY...).  EXPR is evaluated and compared
-against each key in each KEYLIST; the corresponding BODY is evaluated.
-If no clause succeeds, cl-case returns nil.  A single atom may be used in
-place of a KEYLIST of one atom.  A KEYLIST of t or `otherwise' is
-allowed only in the final clause, and matches if no other keys match.
-Key values are compared by `eql'.
+Each clause looks like (KEYLIST BODY...).  EXPR is evaluated and
+compared against each key in each KEYLIST; the corresponding BODY
+is evaluated.  If no clause succeeds, cl-case returns nil.  A
+single non-nil atom may be used in place of a KEYLIST of one
+atom.  A KEYLIST of t or `otherwise' is allowed only in the final
+clause, and matches if no other keys match.  Key values are
+compared by `eql'.
 
 \(fn EXPR (KEYLIST BODY...)...)" nil t)
 
-(put 'cl-case 'lisp-indent-function '1)
+(function-put 'cl-case 'lisp-indent-function '1)
 
 (autoload 'cl-ecase "cl-macs" "\
 Like `cl-case', but error if no case fits.
@@ -344,7 +405,7 @@ Like `cl-case', but error if no case fits.
 
 \(fn EXPR (KEYLIST BODY...)...)" nil t)
 
-(put 'cl-ecase 'lisp-indent-function '1)
+(function-put 'cl-ecase 'lisp-indent-function '1)
 
 (autoload 'cl-typecase "cl-macs" "\
 Evals EXPR, chooses among clauses on that value.
@@ -355,7 +416,7 @@ final clause, and matches if no other keys match.
 
 \(fn EXPR (TYPE BODY...)...)" nil t)
 
-(put 'cl-typecase 'lisp-indent-function '1)
+(function-put 'cl-typecase 'lisp-indent-function '1)
 
 (autoload 'cl-etypecase "cl-macs" "\
 Like `cl-typecase', but error if no case fits.
@@ -363,7 +424,7 @@ Like `cl-typecase', but error if no case fits.
 
 \(fn EXPR (TYPE BODY...)...)" nil t)
 
-(put 'cl-etypecase 'lisp-indent-function '1)
+(function-put 'cl-etypecase 'lisp-indent-function '1)
 
 (autoload 'cl-block "cl-macs" "\
 Define a lexically-scoped block named NAME.
@@ -377,7 +438,7 @@ called from BODY.
 
 \(fn NAME &rest BODY)" nil t)
 
-(put 'cl-block 'lisp-indent-function '1)
+(function-put 'cl-block 'lisp-indent-function '1)
 
 (autoload 'cl-return "cl-macs" "\
 Return from the block named nil.
@@ -394,15 +455,15 @@ This is compatible with Common Lisp, but note that `defun' and
 
 \(fn NAME &optional RESULT)" nil t)
 
-(put 'cl-return-from 'lisp-indent-function '1)
+(function-put 'cl-return-from 'lisp-indent-function '1)
 
 (autoload 'cl-loop "cl-macs" "\
 The Common Lisp `loop' macro.
 Valid clauses include:
   For clauses:
-    for VAR from/upfrom/downfrom EXPR1 to/upto/downto/above/below EXPR2 by EXPR3
+    for VAR from/upfrom/downfrom EXPR1 to/upto/downto/above/below EXPR2 [by EXPR3]
     for VAR = EXPR1 then EXPR2
-    for VAR in/on/in-ref LIST by FUNC
+    for VAR in/on/in-ref LIST [by FUNC]
     for VAR across/across-ref ARRAY
     for VAR being:
       the elements of/of-ref SEQUENCE [using (index VAR2)]
@@ -435,14 +496,14 @@ The Common Lisp `do' loop.
 
 \(fn ((VAR INIT [STEP])...) (END-TEST [RESULT...]) BODY...)" nil t)
 
-(put 'cl-do 'lisp-indent-function '2)
+(function-put 'cl-do 'lisp-indent-function '2)
 
 (autoload 'cl-do* "cl-macs" "\
 The Common Lisp `do*' loop.
 
 \(fn ((VAR INIT [STEP])...) (END-TEST [RESULT...]) BODY...)" nil t)
 
-(put 'cl-do* 'lisp-indent-function '2)
+(function-put 'cl-do* 'lisp-indent-function '2)
 
 (autoload 'cl-dolist "cl-macs" "\
 Loop over a list.
@@ -452,7 +513,7 @@ An implicit nil block is established around the loop.
 
 \(fn (VAR LIST [RESULT]) BODY...)" nil t)
 
-(put 'cl-dolist 'lisp-indent-function '1)
+(function-put 'cl-dolist 'lisp-indent-function '1)
 
 (autoload 'cl-dotimes "cl-macs" "\
 Loop a certain number of times.
@@ -462,7 +523,7 @@ nil.
 
 \(fn (VAR COUNT [RESULT]) BODY...)" nil t)
 
-(put 'cl-dotimes 'lisp-indent-function '1)
+(function-put 'cl-dotimes 'lisp-indent-function '1)
 
 (autoload 'cl-tagbody "cl-macs" "\
 Execute statements while providing for control transfers to labels.
@@ -477,6 +538,18 @@ Labels have lexical scope and dynamic extent.
 
 \(fn &rest LABELS-OR-STMTS)" nil t)
 
+(autoload 'cl-prog "cl-macs" "\
+Run BODY like a `cl-tagbody' after setting up the BINDINGS.
+Shorthand for (cl-block nil (let BINDINGS (cl-tagbody BODY)))
+
+\(fn BINDINGS &rest BODY)" nil t)
+
+(autoload 'cl-prog* "cl-macs" "\
+Run BODY like a `cl-tagbody' after setting up the BINDINGS.
+Shorthand for (cl-block nil (let* BINDINGS (cl-tagbody BODY)))
+
+\(fn BINDINGS &rest BODY)" nil t)
+
 (autoload 'cl-do-symbols "cl-macs" "\
 Loop over all symbols.
 Evaluate BODY with VAR bound to each interned symbol, or to each symbol
@@ -484,14 +557,14 @@ from OBARRAY.
 
 \(fn (VAR [OBARRAY [RESULT]]) BODY...)" nil t)
 
-(put 'cl-do-symbols 'lisp-indent-function '1)
+(function-put 'cl-do-symbols 'lisp-indent-function '1)
 
 (autoload 'cl-do-all-symbols "cl-macs" "\
 Like `cl-do-symbols', but use the default obarray.
 
 \(fn (VAR [RESULT]) BODY...)" nil t)
 
-(put 'cl-do-all-symbols 'lisp-indent-function '1)
+(function-put 'cl-do-all-symbols 'lisp-indent-function '1)
 
 (autoload 'cl-psetq "cl-macs" "\
 Set SYMs to the values VALs in parallel.
@@ -510,15 +583,20 @@ a `let' form, except that the list of symbols can be computed at run-time.
 
 \(fn SYMBOLS VALUES &rest BODY)" nil t)
 
-(put 'cl-progv 'lisp-indent-function '2)
+(function-put 'cl-progv 'lisp-indent-function '2)
 
 (autoload 'cl-flet "cl-macs" "\
 Make local function definitions.
 Like `cl-labels' but the definitions are not recursive.
+Each binding can take the form (FUNC EXP) where
+FUNC is the function name, and EXP is an expression that returns the
+function value to which it should be bound, or it can take the more common
+form (FUNC ARGLIST BODY...) which is a shorthand
+for (FUNC (lambda ARGLIST BODY)).
 
 \(fn ((FUNC ARGLIST BODY...) ...) FORM...)" nil t)
 
-(put 'cl-flet 'lisp-indent-function '1)
+(function-put 'cl-flet 'lisp-indent-function '1)
 
 (autoload 'cl-flet* "cl-macs" "\
 Make local function definitions.
@@ -526,7 +604,7 @@ Like `cl-flet' but the definitions can refer to previous ones.
 
 \(fn ((FUNC ARGLIST BODY...) ...) FORM...)" nil t)
 
-(put 'cl-flet* 'lisp-indent-function '1)
+(function-put 'cl-flet* 'lisp-indent-function '1)
 
 (autoload 'cl-labels "cl-macs" "\
 Make temporary function bindings.
@@ -535,7 +613,7 @@ in closures will only work if `lexical-binding' is in use.
 
 \(fn ((FUNC ARGLIST BODY...) ...) FORM...)" nil t)
 
-(put 'cl-labels 'lisp-indent-function '1)
+(function-put 'cl-labels 'lisp-indent-function '1)
 
 (autoload 'cl-macrolet "cl-macs" "\
 Make temporary macro definitions.
@@ -543,7 +621,7 @@ This is like `cl-flet', but for macros instead of functions.
 
 \(fn ((NAME ARGLIST BODY...) ...) FORM...)" nil t)
 
-(put 'cl-macrolet 'lisp-indent-function '1)
+(function-put 'cl-macrolet 'lisp-indent-function '1)
 
 (autoload 'cl-symbol-macrolet "cl-macs" "\
 Make symbol macro definitions.
@@ -552,7 +630,7 @@ by EXPANSION, and (setq NAME ...) will act like (setf EXPANSION ...).
 
 \(fn ((NAME EXPANSION) ...) FORM...)" nil t)
 
-(put 'cl-symbol-macrolet 'lisp-indent-function '1)
+(function-put 'cl-symbol-macrolet 'lisp-indent-function '1)
 
 (autoload 'cl-multiple-value-bind "cl-macs" "\
 Collect multiple return values.
@@ -564,7 +642,7 @@ a synonym for (list A B C).
 
 \(fn (SYM...) FORM BODY)" nil t)
 
-(put 'cl-multiple-value-bind 'lisp-indent-function '2)
+(function-put 'cl-multiple-value-bind 'lisp-indent-function '2)
 
 (autoload 'cl-multiple-value-setq "cl-macs" "\
 Collect multiple return values.
@@ -575,7 +653,7 @@ values.  For compatibility, (cl-values A B C) is a synonym for (list A B C).
 
 \(fn (SYM...) FORM)" nil t)
 
-(put 'cl-multiple-value-setq 'lisp-indent-function '1)
+(function-put 'cl-multiple-value-setq 'lisp-indent-function '1)
 
 (autoload 'cl-locally "cl-macs" "\
 Equivalent to `progn'.
@@ -583,11 +661,11 @@ Equivalent to `progn'.
 \(fn &rest BODY)" nil t)
 
 (autoload 'cl-the "cl-macs" "\
-At present this ignores TYPE and is simply equivalent to FORM.
+Return FORM.  If type-checking is enabled, assert that it is of TYPE.
 
 \(fn TYPE FORM)" nil t)
 
-(put 'cl-the 'lisp-indent-function '1)
+(function-put 'cl-the 'lisp-indent-function '1)
 
 (autoload 'cl-declare "cl-macs" "\
 Declare SPECS about the current function while compiling.
@@ -640,7 +718,7 @@ the PLACE is not modified before executing BODY.
 
 \(fn ((PLACE VALUE) ...) BODY...)" nil t)
 
-(put 'cl-letf 'lisp-indent-function '1)
+(function-put 'cl-letf 'lisp-indent-function '1)
 
 (autoload 'cl-letf* "cl-macs" "\
 Temporarily bind to PLACEs.
@@ -649,7 +727,7 @@ rather than all at the end (i.e. like `let*' rather than like `let').
 
 \(fn BINDINGS &rest BODY)" nil t)
 
-(put 'cl-letf* 'lisp-indent-function '1)
+(function-put 'cl-letf* 'lisp-indent-function '1)
 
 (autoload 'cl-callf "cl-macs" "\
 Set PLACE to (FUNC PLACE ARGS...).
@@ -658,7 +736,7 @@ or any generalized variable allowed by `setf'.
 
 \(fn FUNC PLACE &rest ARGS)" nil t)
 
-(put 'cl-callf 'lisp-indent-function '2)
+(function-put 'cl-callf 'lisp-indent-function '2)
 
 (autoload 'cl-callf2 "cl-macs" "\
 Set PLACE to (FUNC ARG1 PLACE ARGS...).
@@ -666,7 +744,19 @@ Like `cl-callf', but PLACE is the second argument of FUNC, not the first.
 
 \(fn FUNC ARG1 PLACE ARGS...)" nil t)
 
-(put 'cl-callf2 'lisp-indent-function '3)
+(function-put 'cl-callf2 'lisp-indent-function '3)
+
+(autoload 'cl-defsubst "cl-macs" "\
+Define NAME as a function.
+Like `defun', except the function is automatically declared `inline' and
+the arguments are immutable.
+ARGLIST allows full Common Lisp conventions, and BODY is implicitly
+surrounded by (cl-block NAME ...).
+The function's arguments should be treated as immutable.
+
+\(fn NAME ARGLIST [DOCSTRING] BODY...)" nil t)
+
+(function-put 'cl-defsubst 'lisp-indent-function '2)
 
 (autoload 'cl-defstruct "cl-macs" "\
 Define a struct type.
@@ -688,27 +778,28 @@ non-nil value, that slot cannot be set via `setf'.
 
 \(fn NAME SLOTS...)" nil t)
 
-(put 'cl-defstruct 'doc-string-elt '2)
+(function-put 'cl-defstruct 'doc-string-elt '2)
 
-(put 'cl-defstruct 'lisp-indent-function '1)
+(function-put 'cl-defstruct 'lisp-indent-function '1)
 
-(autoload 'cl-deftype "cl-macs" "\
-Define NAME as a new data type.
-The type name can then be used in `cl-typecase', `cl-check-type', etc.
+(autoload 'cl-struct--pcase-macroexpander "cl-macs" "\
+Pcase patterns to match cl-structs.
+Elements of FIELDS can be of the form (NAME PAT) in which case the contents of
+field NAME is matched against PAT, or they can be of the form NAME which
+is a shorthand for (NAME NAME).
 
-\(fn NAME ARGLIST &rest BODY)" nil t)
+\(fn TYPE &rest FIELDS)" nil nil)
 
-(put 'cl-deftype 'doc-string-elt '3)
+(define-symbol-prop 'cl-struct--pcase-macroexpander 'edebug-form-spec '(sexp &rest [&or (sexp pcase-PAT) sexp]))
 
-(put 'cl-deftype 'lisp-indent-function '2)
+(define-symbol-prop 'cl-struct 'pcase-macroexpander #'cl-struct--pcase-macroexpander)
 
 (autoload 'cl-typep "cl-macs" "\
-Check that OBJECT is of type TYPE.
-TYPE is a Common Lisp-style type specifier.
 
-\(fn OBJECT TYPE)" nil nil)
 
-(eval-and-compile (put 'cl-typep 'compiler-macro #'cl--compiler-macro-typep))
+\(fn VAL TYPE)" nil nil)
+
+(eval-and-compile (function-put 'cl-typep 'compiler-macro #'cl-typep--inliner))
 
 (autoload 'cl-check-type "cl-macs" "\
 Verify that FORM is of type TYPE; signal an error if not.
@@ -739,7 +830,7 @@ and then returning foo.
 
 \(fn FUNC ARGS &rest BODY)" nil t)
 
-(put 'cl-define-compiler-macro 'lisp-indent-function '2)
+(function-put 'cl-define-compiler-macro 'lisp-indent-function '2)
 
 (autoload 'cl-compiler-macroexpand "cl-macs" "\
 Like `macroexpand', but for compiler macros.
@@ -749,26 +840,24 @@ macro that returns its `&whole' argument.
 
 \(fn FORM)" nil nil)
 
-(autoload 'cl-defsubst "cl-macs" "\
-Define NAME as a function.
-Like `defun', except the function is automatically declared `inline' and
-the arguments are immutable.
-ARGLIST allows full Common Lisp conventions, and BODY is implicitly
-surrounded by (cl-block NAME ...).
-The function's arguments should be treated as immutable.
-
-\(fn NAME ARGLIST [DOCSTRING] BODY...)" nil t)
-
-(put 'cl-defsubst 'lisp-indent-function '2)
-
 (autoload 'cl--compiler-macro-adjoin "cl-macs" "\
 
 
 \(fn FORM A LIST &rest KEYS)" nil nil)
 
+(autoload 'cl-deftype "cl-macs" "\
+Define NAME as a new data type.
+The type name can then be used in `cl-typecase', `cl-check-type', etc.
+
+\(fn NAME ARGLIST &rest BODY)" nil t)
+
+(function-put 'cl-deftype 'doc-string-elt '3)
+
+(function-put 'cl-deftype 'lisp-indent-function '2)
+
 ;;;***
 
-;;;### (autoloads nil "cl-seq" "cl-seq.el" "5b212bdeb4affa862cbfd49e4e3e8bb2")
+;;;### (autoloads nil "cl-seq" "cl-seq.el" "bfa46459c2a3a1a8c255e21619c23f70")
 ;;; Generated autoloads from cl-seq.el
 
 (autoload 'cl-reduce "cl-seq" "\
@@ -776,6 +865,16 @@ Reduce two-argument FUNCTION across SEQ.
 
 Keywords supported:  :start :end :from-end :initial-value :key
 
+Return the result of calling FUNCTION with the first and the
+second element of SEQ, then calling FUNCTION with that result and
+the third element of SEQ, then with that result and the fourth
+element of SEQ, etc.
+
+If :INITIAL-VALUE is specified, it is added to the front of SEQ.
+If SEQ is empty, return :INITIAL-VALUE and FUNCTION is not
+called.
+
+
 \(fn FUNCTION SEQ [KEYWORD VALUE]...)" nil nil)
 
 (autoload 'cl-fill "cl-seq" "\
@@ -1029,7 +1128,7 @@ Keywords supported:  :test :test-not :key
 
 \(fn ITEM LIST [KEYWORD VALUE]...)" nil nil)
 
-(eval-and-compile (put 'cl-member 'compiler-macro #'cl--compiler-macro-member))
+(eval-and-compile (function-put 'cl-member 'compiler-macro #'cl--compiler-macro-member))
 
 (autoload 'cl-member-if "cl-seq" "\
 Find the first item satisfying PREDICATE in LIST.
@@ -1059,7 +1158,7 @@ Keywords supported:  :test :test-not :key
 
 \(fn ITEM LIST [KEYWORD VALUE]...)" nil nil)
 
-(eval-and-compile (put 'cl-assoc 'compiler-macro #'cl--compiler-macro-assoc))
+(eval-and-compile (function-put 'cl-assoc 'compiler-macro #'cl--compiler-macro-assoc))
 
 (autoload 'cl-assoc-if "cl-seq" "\
 Find the first item whose car satisfies PREDICATE in LIST.
