@@ -1,16 +1,16 @@
 /* Primitive operations on floating point for GNU Emacs Lisp interpreter.
 
-Copyright (C) 1988, 1993-1994, 1999, 2001-2014 Free Software Foundation, Inc.
+Copyright (C) 1988, 1993-1994, 1999, 2001-2018 Free Software Foundation,
+Inc.
 
-Author: Wolfgang Rupprecht
-(according to ack.texi)
+Author: Wolfgang Rupprecht (according to ack.texi)
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+the Free Software Foundation, either version 3 of the License, or (at
+your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
+along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 /* C89 requires only the following math.h functions, and Emacs omits
@@ -36,7 +36,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
    isnormal, isunordered, lgamma, log1p, *log2 [via (log X 2)], *logb
    (approximately), lrint/llrint, lround/llround, nan, nearbyint,
    nextafter, nexttoward, remainder, remquo, *rint, round, scalbln,
-   scalbn, signbit, tgamma, trunc.
+   scalbn, signbit, tgamma, *trunc.
  */
 
 #include <config.h>
@@ -45,13 +45,14 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <math.h>
 
-/* 'isfinite' and 'isnan' cause build failures on Solaris 10 with the
-   bundled GCC in c99 mode.  Work around the bugs with simple
-   implementations that are good enough.  */
-#undef isfinite
-#define isfinite(x) ((x) - (x) == 0)
-#undef isnan
-#define isnan(x) ((x) != (x))
+#include <count-leading-zeros.h>
+
+#ifndef isfinite
+# define isfinite(x) ((x) - (x) == 0)
+#endif
+#ifndef isnan
+# define isnan(x) ((x) != (x))
+#endif
 
 /* Check that X is a floating point number.  */
 
@@ -67,10 +68,7 @@ double
 extract_float (Lisp_Object num)
 {
   CHECK_NUMBER_OR_FLOAT (num);
-
-  if (FLOATP (num))
-    return XFLOAT_DATA (num);
-  return (double) XINT (num);
+  return XFLOATINT (num);
 }
 
 /* Trig functions.  */
@@ -148,7 +146,12 @@ DEFUN ("isnan", Fisnan, Sisnan, 1, 1, 0,
   return isnan (XFLOAT_DATA (x)) ? Qt : Qnil;
 }
 
-#ifdef HAVE_COPYSIGN
+/* Although the substitute does not work on NaNs, it is good enough
+   for platforms lacking the signbit macro.  */
+#ifndef signbit
+# define signbit(x) ((x) < 0 || (IEEE_FLOATING_POINT && !(x) && 1 / (x) < 0))
+#endif
+
 DEFUN ("copysign", Fcopysign, Scopysign, 2, 2, 0,
        doc: /* Copy sign of X2 to value of X1, and return the result.
 Cause an error if X1 or X2 is not a float.  */)
@@ -162,9 +165,10 @@ Cause an error if X1 or X2 is not a float.  */)
   f1 = XFLOAT_DATA (x1);
   f2 = XFLOAT_DATA (x2);
 
-  return make_float (copysign (f1, f2));
+  /* Use signbit instead of copysign, to avoid calling make_float when
+     the result is X1.  */
+  return signbit (f1) != signbit (f2) ? make_float (-f1) : x1;
 }
-#endif
 
 DEFUN ("frexp", Ffrexp, Sfrexp, 1, 1, 0,
        doc: /* Get significand and exponent of a floating point number.
@@ -178,20 +182,20 @@ The function returns the cons cell (SGNFCAND . EXP).
 If X is zero, both parts (SGNFCAND and EXP) are zero.  */)
   (Lisp_Object x)
 {
-  double f = XFLOATINT (x);
+  double f = extract_float (x);
   int exponent;
   double sgnfcand = frexp (f, &exponent);
   return Fcons (make_float (sgnfcand), make_number (exponent));
 }
 
-DEFUN ("ldexp", Fldexp, Sldexp, 1, 2, 0,
-       doc: /* Construct number X from significand SGNFCAND and exponent EXP.
-Returns the floating point value resulting from multiplying SGNFCAND
-(the significand) by 2 raised to the power of EXP (the exponent).   */)
+DEFUN ("ldexp", Fldexp, Sldexp, 2, 2, 0,
+       doc: /* Return SGNFCAND * 2**EXPONENT, as a floating point number.
+EXPONENT must be an integer.   */)
   (Lisp_Object sgnfcand, Lisp_Object exponent)
 {
   CHECK_NUMBER (exponent);
-  return make_float (ldexp (XFLOATINT (sgnfcand), XINT (exponent)));
+  int e = min (max (INT_MIN, XINT (exponent)), INT_MAX);
+  return make_float (ldexp (extract_float (sgnfcand), e));
 }
 
 DEFUN ("exp", Fexp, Sexp, 1, 1, 0,
@@ -207,8 +211,6 @@ DEFUN ("expt", Fexpt, Sexpt, 2, 2, 0,
        doc: /* Return the exponential ARG1 ** ARG2.  */)
   (Lisp_Object arg1, Lisp_Object arg2)
 {
-  double f1, f2, f3;
-
   CHECK_NUMBER_OR_FLOAT (arg1);
   CHECK_NUMBER_OR_FLOAT (arg2);
   if (INTEGERP (arg1)     /* common lisp spec */
@@ -232,10 +234,7 @@ DEFUN ("expt", Fexpt, Sexpt, 2, 2, 0,
       XSETINT (val, acc);
       return val;
     }
-  f1 = FLOATP (arg1) ? XFLOAT_DATA (arg1) : XINT (arg1);
-  f2 = FLOATP (arg2) ? XFLOAT_DATA (arg2) : XINT (arg2);
-  f3 = pow (f1, f2);
-  return make_float (f3);
+  return make_float (pow (XFLOATINT (arg1), XFLOATINT (arg2)));
 }
 
 DEFUN ("log", Flog, Slog, 1, 2, 0,
@@ -298,28 +297,46 @@ DEFUN ("float", Ffloat, Sfloat, 1, 1, 0,
     return arg;
 }
 
+static int
+ecount_leading_zeros (EMACS_UINT x)
+{
+  return (EMACS_UINT_WIDTH == UINT_WIDTH ? count_leading_zeros (x)
+	  : EMACS_UINT_WIDTH == ULONG_WIDTH ? count_leading_zeros_l (x)
+	  : count_leading_zeros_ll (x));
+}
+
 DEFUN ("logb", Flogb, Slogb, 1, 1, 0,
        doc: /* Returns largest integer <= the base 2 log of the magnitude of ARG.
 This is the same as the exponent of a float.  */)
   (Lisp_Object arg)
 {
-  Lisp_Object val;
   EMACS_INT value;
-  double f = extract_float (arg);
+  CHECK_NUMBER_OR_FLOAT (arg);
 
-  if (f == 0.0)
-    value = MOST_NEGATIVE_FIXNUM;
-  else if (isfinite (f))
+  if (FLOATP (arg))
     {
-      int ivalue;
-      frexp (f, &ivalue);
-      value = ivalue - 1;
+      double f = XFLOAT_DATA (arg);
+
+      if (f == 0)
+	value = MOST_NEGATIVE_FIXNUM;
+      else if (isfinite (f))
+	{
+	  int ivalue;
+	  frexp (f, &ivalue);
+	  value = ivalue - 1;
+	}
+      else
+	value = MOST_POSITIVE_FIXNUM;
     }
   else
-    value = MOST_POSITIVE_FIXNUM;
+    {
+      EMACS_INT i = eabs (XINT (arg));
+      value = (i == 0
+	       ? MOST_NEGATIVE_FIXNUM
+	       : EMACS_UINT_WIDTH - 1 - ecount_leading_zeros (i));
+    }
 
-  XSETINT (val, value);
-  return val;
+  return make_number (value);
 }
 
 
@@ -333,75 +350,60 @@ rounding_driver (Lisp_Object arg, Lisp_Object divisor,
 {
   CHECK_NUMBER_OR_FLOAT (arg);
 
-  if (! NILP (divisor))
+  double d;
+  if (NILP (divisor))
+    {
+      if (! FLOATP (arg))
+	return arg;
+      d = XFLOAT_DATA (arg);
+    }
+  else
     {
-      EMACS_INT i1, i2;
-
       CHECK_NUMBER_OR_FLOAT (divisor);
-
-      if (FLOATP (arg) || FLOATP (divisor))
+      if (!FLOATP (arg) && !FLOATP (divisor))
 	{
-	  double f1, f2;
-
-	  f1 = FLOATP (arg) ? XFLOAT_DATA (arg) : XINT (arg);
-	  f2 = (FLOATP (divisor) ? XFLOAT_DATA (divisor) : XINT (divisor));
-	  if (! IEEE_FLOATING_POINT && f2 == 0)
+	  if (XINT (divisor) == 0)
 	    xsignal0 (Qarith_error);
-
-	  f1 = (*double_round) (f1 / f2);
-	  if (FIXNUM_OVERFLOW_P (f1))
-	    xsignal3 (Qrange_error, build_string (name), arg, divisor);
-	  arg = make_number (f1);
-	  return arg;
+	  return make_number (int_round2 (XINT (arg), XINT (divisor)));
 	}
 
-      i1 = XINT (arg);
-      i2 = XINT (divisor);
-
-      if (i2 == 0)
+      double f1 = FLOATP (arg) ? XFLOAT_DATA (arg) : XINT (arg);
+      double f2 = FLOATP (divisor) ? XFLOAT_DATA (divisor) : XINT (divisor);
+      if (! IEEE_FLOATING_POINT && f2 == 0)
 	xsignal0 (Qarith_error);
-
-      XSETINT (arg, (*int_round2) (i1, i2));
-      return arg;
+      d = f1 / f2;
     }
 
-  if (FLOATP (arg))
+  /* Round, coarsely test for fixnum overflow before converting to
+     EMACS_INT (to avoid undefined C behavior), and then exactly test
+     for overflow after converting (as FIXNUM_OVERFLOW_P is inaccurate
+     on floats).  */
+  double dr = double_round (d);
+  if (fabs (dr) < 2 * (MOST_POSITIVE_FIXNUM + 1))
     {
-      double d = (*double_round) (XFLOAT_DATA (arg));
-      if (FIXNUM_OVERFLOW_P (d))
-	xsignal2 (Qrange_error, build_string (name), arg);
-      arg = make_number (d);
+      EMACS_INT ir = dr;
+      if (! FIXNUM_OVERFLOW_P (ir))
+	return make_number (ir);
     }
-
-  return arg;
+  xsignal2 (Qrange_error, build_string (name), arg);
 }
 
-/* With C's /, the result is implementation-defined if either operand
-   is negative, so take care with negative operands in the following
-   integer functions.  */
-
 static EMACS_INT
 ceiling2 (EMACS_INT i1, EMACS_INT i2)
 {
-  return (i2 < 0
-	  ? (i1 < 0  ?  ((-1 - i1) / -i2) + 1  :  - (i1 / -i2))
-	  : (i1 <= 0  ?  - (-i1 / i2)  :  ((i1 - 1) / i2) + 1));
+  return i1 / i2 + ((i1 % i2 != 0) & ((i1 < 0) == (i2 < 0)));
 }
 
 static EMACS_INT
 floor2 (EMACS_INT i1, EMACS_INT i2)
 {
-  return (i2 < 0
-	  ? (i1 <= 0  ?  -i1 / -i2  :  -1 - ((i1 - 1) / -i2))
-	  : (i1 < 0  ?  -1 - ((-1 - i1) / i2)  :  i1 / i2));
+  return i1 / i2 - ((i1 % i2 != 0) & ((i1 < 0) != (i2 < 0)));
 }
 
 static EMACS_INT
 truncate2 (EMACS_INT i1, EMACS_INT i2)
 {
-  return (i2 < 0
-	  ? (i1 < 0  ?  -i1 / -i2  :  - (i1 / -i2))
-	  : (i1 < 0  ?  - (-i1 / i2)  :  i1 / i2));
+  return i1 / i2;
 }
 
 static EMACS_INT
@@ -433,11 +435,15 @@ emacs_rint (double d)
 }
 #endif
 
+#ifdef HAVE_TRUNC
+#define emacs_trunc trunc
+#else
 static double
-double_identity (double d)
+emacs_trunc (double d)
 {
-  return d;
+  return (d < 0 ? ceil : floor) (d);
 }
+#endif
 
 DEFUN ("ceiling", Fceiling, Sceiling, 1, 2, 0,
        doc: /* Return the smallest integer no less than ARG.
@@ -463,7 +469,7 @@ With optional DIVISOR, return the nearest integer to ARG/DIVISOR.
 
 Rounding a value equidistant between two integers may choose the
 integer closer to zero, or it may prefer an even integer, depending on
-your machine.  For example, \(round 2.5\) can return 3 on some
+your machine.  For example, (round 2.5) can return 3 on some
 systems, but 2 on others.  */)
   (Lisp_Object arg, Lisp_Object divisor)
 {
@@ -476,7 +482,7 @@ Rounds ARG toward zero.
 With optional DIVISOR, truncate ARG/DIVISOR.  */)
   (Lisp_Object arg, Lisp_Object divisor)
 {
-  return rounding_driver (arg, divisor, double_identity, truncate2,
+  return rounding_driver (arg, divisor, emacs_trunc, truncate2,
 			  "truncate");
 }
 
@@ -500,20 +506,22 @@ fmod_float (Lisp_Object x, Lisp_Object y)
 
 DEFUN ("fceiling", Ffceiling, Sfceiling, 1, 1, 0,
        doc: /* Return the smallest integer no less than ARG, as a float.
-\(Round toward +inf.\)  */)
+\(Round toward +inf.)  */)
   (Lisp_Object arg)
 {
-  double d = extract_float (arg);
+  CHECK_FLOAT (arg);
+  double d = XFLOAT_DATA (arg);
   d = ceil (d);
   return make_float (d);
 }
 
 DEFUN ("ffloor", Fffloor, Sffloor, 1, 1, 0,
        doc: /* Return the largest integer no greater than ARG, as a float.
-\(Round towards -inf.\)  */)
+\(Round toward -inf.)  */)
   (Lisp_Object arg)
 {
-  double d = extract_float (arg);
+  CHECK_FLOAT (arg);
+  double d = XFLOAT_DATA (arg);
   d = floor (d);
   return make_float (d);
 }
@@ -522,21 +530,20 @@ DEFUN ("fround", Ffround, Sfround, 1, 1, 0,
        doc: /* Return the nearest integer to ARG, as a float.  */)
   (Lisp_Object arg)
 {
-  double d = extract_float (arg);
+  CHECK_FLOAT (arg);
+  double d = XFLOAT_DATA (arg);
   d = emacs_rint (d);
   return make_float (d);
 }
 
 DEFUN ("ftruncate", Fftruncate, Sftruncate, 1, 1, 0,
        doc: /* Truncate a floating point number to an integral float value.
-Rounds the value toward zero.  */)
+\(Round toward zero.)  */)
   (Lisp_Object arg)
 {
-  double d = extract_float (arg);
-  if (d >= 0.0)
-    d = floor (d);
-  else
-    d = ceil (d);
+  CHECK_FLOAT (arg);
+  double d = XFLOAT_DATA (arg);
+  d = emacs_trunc (d);
   return make_float (d);
 }
 
@@ -550,9 +557,7 @@ syms_of_floatfns (void)
   defsubr (&Ssin);
   defsubr (&Stan);
   defsubr (&Sisnan);
-#ifdef HAVE_COPYSIGN
   defsubr (&Scopysign);
-#endif
   defsubr (&Sfrexp);
   defsubr (&Sldexp);
   defsubr (&Sfceiling);
